/***************************************************************************
*
* LS_ATIME - List last access date and time of files.
*
* Copyright (c) 1990 by Frank R. Slootweg. Version 1.0.
*
*	List last access date and time of files, as recorded by DCOMPRES in
*	the file INDEX.CMP, in the current or specified directory.
*	Its main purpose, at least for the author, is to determine which
*	files have not been accessed for a long time or have not been
*	accessed at all and therefore are candidates for removal.
*
* Usage : LS_ATIME [<d>:][<path>]
*
*	d:	the drive that contains the file(s) to be displayed.
*	path	the path to the file(s) to be displayed.
*
*	I.e. similar to the DIR command. However no wildcards can be used.
*
* Example output :
*
*	DOIT.BAT       2-10-90  18:53:15.78
*	FILE2          2-10-90  18:48:55.82
*	FILE1          2-10-90  18:53:15.72
*	DATE.EXE       2-11-90  16:58:12.47
*
* Revision History :
*
* 2-12-90  1.0	Initial release. Compatible with DCOMPRES Version 1.1.
*
* Written in/for Turbo C 2.0. Link(ed) with: Debug: Source debugging: None.
*
* Note: The program DCOMPRES is part of the PC Magazine, Vol. 8, No.21
*	Utility DCOMPRES/PCMANAGE written by Ross M. Greenberg of Ziff
*	Communications Co.
*
* Usage and technical disclaimers :
*
*	- This program may become obsolete in the future as I will probably
*	  contact the author of the DCOMPRES/PCMANAGE utility and ask him to
*	  incorporate the functionality of LS_ATIME in DCOMPRES/PCMANAGE.
*	- The program expects the number of files (entries) in the index
*	  file INDEX.CMP to be found in the integer (dw or int) "num_files"
*	  at the beginning of the index file. See DCOMPRES.ASM and
*	  PCMANAGE.C for details.
*	- The beginning of the binary tree index is expected to be fixed
*	  (sizeof(num_files) + MAX_PLEN + MAX_FLEN bytes from the beginning
*	  of the index file). See DCOMPRES.ASM and PCMANAGE.C for details.
*
* License :
*
*	LS_ATIME (c) Copyright 1990 by Frank R. Slootweg. All Rights Reserved.
*
*	LS_ATIME is free, but it is a copyrighted work. License is hereby
*	granted to distribute it for personal, non-commercial use. You may
*	use it yourself, give it to your friends or co-workers, or
*	distribute it for a cost-based fee ($10 or less) as part of a user's
*	group or bulletin board service.
*
*	You are free to use, copy and distribute LS_ATIME providing:
*
*	- No fee is charged for use, copying or distribution.
*
*	- The program is supplied in its in its original, unmodified form,
*	  which includes this documentation, source, executable, the
*	  copyright notice, this license and the warrenty disclaimer.
*
*	You may modify the source for your own personal use, but you may not
*	distribute the modified source or the resulting modified executable
*	without prior approval from the author.
*
*	Please direct your inquiries, complaints, suggestions, etc., to :
*
*	Frank R. Slootweg
*	Rozenstraat 16
*	2431 XE  Noorden
*	The Netherlands
*
*	Internet   : franks@hpuamsa.neth.hp.com
*	UUCP       : {hplabs|mcvax|hp4nl}!hpuamsa!franks
*	CompuServe : >INTERNET:franks@hpuamsa.neth.hp.com
*
* Warrenty disclaimer:
*
*	The author hereby disclaims all warranties relating to this
*	software, whether express or implied, including without limitation
*	any implied warranties of merchantability or fitness for a
*	particular purpose.  The author will not be liable for any special,
*	incidental, consequential, indirect or similar damages due to loss
*	of data or any other reason, even if the author has been advised of
*	the possibility of such damages.  In no event shall the	author's
*	liability for any damages ever exceed the price paid for the license
*	to use software, regardless of the form of the claim.
*	The person using the software bears all risk as to the quality and
*	performance of the software.
*
**************************************************************************/

/* Standard Include Files : */

#include <dos.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>

#define MAX_PLEN	64	/* Maximum path name length. */
#define MAX_FLEN	14	/* Maximum file name length. */

/* Structure for entry in index file INDEX.CMP : */

struct INDEX {
   int		fname_len;
   char		filename[MAX_FLEN];
   unsigned	left_ptr;
   unsigned	right_ptr;
   struct	date da;
   struct	time ti;
   char		status;
   char		access_cnt;
};

extern	int _fmode = O_BINARY;	/* Access index file in binary mode. */

/* Main program : */
main(argc, argv)
int  argc;
char *argv[];
{
   int		length, fd, num_files, f, c;
   long		status;
   struct	INDEX index;
   char		indexfile[MAX_PLEN + MAX_FLEN], char_year[4];

   length = 0;	/* Set path length to 0. */

/* If argument (path) specified then copy it to "indexfile" :		 */
   if (argc == 2) {
      length = strlen(argv[1]);		/* Determine path length.	 */
      if (length > MAX_PLEN) {          /* Check for maximum path length.*/
	 fprintf(stderr, "Path too long (maximum %d chars)\n", MAX_PLEN);
	 exit(1);
      }
      strcpy(&indexfile, argv[1]);	/* Copy path to "indexfile".	 */
      if (indexfile[length - 1] != '\\') {	/* If no trailing "\"	 */
	 strcpy(&indexfile[length],"\\");	/* then set trailing "\" */
	 length++;				/* and adjust length.	 */
      }
   }

/* Append "index.cmp" to default (current directory) or specified path : */
   strcpy(&indexfile[length], "index.cmp");

/* Open the index file : */
   fd = open(indexfile, O_RDONLY);
   status = fd;
   errorck(status, indexfile);		/* Check for errors. */

/* Get number of files in the index : */
   status = read(fd, &num_files , sizeof(num_files));
   errorck(status, "read num_files");

/* Seek to the beginning of the index : */
   status = lseek(fd, (long)(sizeof(num_files)+MAX_PLEN+MAX_FLEN), SEEK_SET);
   errorck (status, "lseek");

/* Read the entries and print the filename and last access date and time :*/
   for (f = 0 ; f < num_files ; f++) {
   /* Read entry :	  */
      status = read(fd, &index , sizeof(index));
      errorck(status, "read struct index");

      printf("%s", index.filename);	/* Print filename. */
      for (c = index.fname_len ; c < 14 ; c++)
	 putchar(' ');

   /* Print last access date : */
      sprintf(char_year, "%4d", index.da.da_year);
      printf("%2d-%2d-%s  ", index.da.da_mon, index.da.da_day, &char_year[2]);

   /* Print last access time : */
      printf("%2d:%2d:%2d", index.ti.ti_hour, index.ti.ti_min, index.ti.ti_sec);
      printf(".%d\n", index.ti.ti_hund);
   }

/* Close the index file : */
   close(fd);
}

int errorck(status, message)	/* Error checking and printing function. */
long status;
char *message;
{
   if (status < 0) {
      perror(message);
      exit(1);
   }
}
