/*	MCopy by Russell Wallace - see instructions for details
	Compile with small memory model
	Link with wildargs.obj and acs.lib
*/

#include	<ctype.h>
#include	<dos.h>
#include	<stdio.h>
#include	<string.h>
#include	<io.h>
#include	<fcntl.h>
#include	<dir.h>
#include	<rwlib.h>

int restore;
int noBreakup;
int floppyDrive = 1;
int floppyno;
char mcPathName[] = "?:\\MCFILE";
char floppyPathName[16] = "?:\\";
char floppyWildCard[] = "?:\\*.*";
int lastRestoreFloppy;
int clustersize;

void insertFloppy ()
{
	printf ("Please insert floppy disk number %d and press RETURN",floppyno);
	do
		;
	while (getchar () != '\n');
}

void insertBackupFloppy ()
{
	floppyno++;
	insertFloppy ();
	struct ffblk ffblk;
	if (!findfirst (floppyWildCard,&ffblk,0))
		do
		{
			strcpy (floppyPathName+3,ffblk.ff_name);
			unlink (floppyPathName);
		}
		while (!findnext (&ffblk));
}

long getFloppySpace ()
{
	struct dfree dtable;
	getdfree (floppyDrive,&dtable);
	clustersize = dtable.df_bsec * dtable.df_sclus;
	return (unsigned long)(dtable.df_avail - 1) *
			 (unsigned long) dtable.df_bsec *
			 (unsigned long) dtable.df_sclus;
}

void backupfiles (int argc,char **argv)
{
	insertBackupFloppy ();
	long floppyBytes = getFloppySpace ();

	int mcFile;

	for (int i=1 ; i<argc ; i++)
		if (argv[i][0] != '-')
		{
			int src = Open (argv[i]);
			struct ftime ftime;
			getftime (src,&ftime);
			long fileBytes = filelength (src);
			int doneThisFile = FALSE;
			while (fileBytes || !doneThisFile)
			{
				doneThisFile = TRUE;
				if (noBreakup && (floppyBytes < fileBytes))
				{
					mcFile = Create (mcPathName);
					Write (mcFile,&floppyno,sizeof (floppyno));
					close (mcFile);
					insertBackupFloppy ();
					floppyBytes = getFloppySpace ();
					if (floppyBytes < fileBytes)
						perr ("File too big to fit on one floppy, redo without -B");
				}
				if (!floppyBytes)
				{
					mcFile = Create (mcPathName);
					Write (mcFile,&floppyno,sizeof (floppyno));
					close (mcFile);
					insertBackupFloppy ();
					floppyBytes = getFloppySpace ();
				}
				strcpy (floppyPathName+3,getfilename (argv[i]));
				int dest = Create (floppyPathName);
				long copyBytes = Min (fileBytes,floppyBytes);
				fileCopy (dest,src,copyBytes);
				setftime (dest,&ftime);
				close (dest);
				puts (argv[i]);
				fileBytes -= copyBytes;
				if (copyBytes % clustersize)
					copyBytes += clustersize - copyBytes % clustersize;
				floppyBytes -= copyBytes;
			}
			close (src);
		}

	mcFile = Create (mcPathName);
	floppyno = ~floppyno;
	Write (mcFile,&floppyno,sizeof (floppyno));
}

void getRestoreFloppy ()
{
	floppyno++;
REDO:
	insertFloppy ();
	int mcFile = open (mcPathName,O_RDWR | O_BINARY);
	if (mcFile < 0)
	{
		printf ("This disk was not backed up with MCopy.\n");
		goto REDO;
	}
	int mcfile_floppyno;
	Read (mcFile,&mcfile_floppyno,sizeof (mcfile_floppyno));
	if (mcfile_floppyno < 0)
	{
		mcfile_floppyno = ~mcfile_floppyno;
		lastRestoreFloppy = TRUE;
	}
	if (mcfile_floppyno != floppyno)
	{
		printf ("This is floppy disk number %d.\n",mcfile_floppyno);
		lastRestoreFloppy = FALSE;
		goto REDO;
	}
}

void restorefiles ()
{
	struct ffblk ffblk;
	static tree fnames = { (int (*)(void *,void *))strcmp };
	static tree oldfnames = { (int (*)(void *,void *))strcmp };

	do
	{
		getRestoreFloppy ();
		findfirst (floppyWildCard,&ffblk,0);
		do
			if (strcmp (ffblk.ff_name,"MCFILE"))
			{
				strcpy (floppyPathName+3,ffblk.ff_name);
				int src = Open (floppyPathName);
				struct ftime ftime;
				getftime (src,&ftime);
				int dest;
				if (oldfnames[ffblk.ff_name])
					dest = OpenAppend (ffblk.ff_name);
				else
					dest = Create (ffblk.ff_name);
				fileCopy (dest,src,filelength (src));
				setftime (dest,&ftime);
				close (dest);
				close (src);
				fnames += strdup (ffblk.ff_name);
				puts (ffblk.ff_name);
			}
		while (!findnext (&ffblk));
		oldfnames.free ();
		oldfnames.t = fnames.t;
		fnames.t = 0;
	}
	while (!lastRestoreFloppy);
}

main (int argc,char **argv)
{
	if (argc < 2 || !strcmp (argv[1],"?"))
		perr (
"MultiCopy v3.0 by Russell Wallace  " __DATE__ "\n"
"Usage: MCopy <files>\n"
"Copies indicated files to floppy disk\n"
"\n"
"Options:\n"
"-D\tUse Drive B: instead of A: (A: is default)\n"
"-R\tRestore files instead of backing up\n"
"-B\tDon't break up files over multiple disks\n"
"\n"
"Files will be scattered over multiple disks if they won't all fit on the first\n"
"disk. If -B is not used files will be broken into several parts. These will be\n"
"put back together by MCopy -R but in any case the parts can be appended.\n"
"Files in the root directory of the floppy disk will be deleted. Disks must be\n"
"formatted before files can be placed on them with MCopy. The file MCFILE\n"
"which is created on each target floppy contains information about how many\n"
"disks there are in total and which disks broken-up files are on.\n"
"\n"
"MCopy -R restores all files from floppy disk to current directory. No other\n"
"arguments are needed."
);
	for (int i=1 ; i<argc ; i++)
	{
		if (argv[i][0] == '-')
		{
			for (char *s=argv[i]+1 ; *s ; s++)
				switch (tolower (*s))
				{
					case 'r':
						restore = TRUE;
						break;
					case 'b':
						noBreakup = TRUE;
						break;
					case 'd':
						floppyDrive = 2;
						break;
					default:
						perr ("Invalid option -%c",*s);
				}
		}
	}
	mcPathName[0] = floppyPathName[0] = floppyDrive - 1 + 'A';
	floppyWildCard[0] = floppyDrive - 1 + 'A';
	if (restore)
		restorefiles ();
	else
		backupfiles (argc,argv);
	return 0;
}
