/*
 *  split  --  UNIX like "split" for DOS
 *
 *	Gregory Margo	08/17/87	initial
 *
 *
 *  (C) Copyright 1987, 1988 Gregory H. Margo  All Rights Reserved.
 *
 *  Released to the Public Domain for personal use only.
 *  May not be included in any commercial product.
 *
 *  Support PUBLIC DOMAIN software, not BEGWARE!
 *
 *
 *
 *  Compiled under Microsoft C V4.0.
 *    ==> cl -G0 -Ot -DLINT_ARGS -c split.c
 *    ==> link split,split,NUL,/NOI;
 */

#include	<stdio.h>
#include	<ctype.h>
#include	<string.h>

#ifndef	LPNMAX
#define	LPNMAX	128
#endif
#define	LINE_SIZE	512
#define	BASE_NAME_LEN	8
#define	SHIFT	argc--; argv++
#define	NUMLINESDEFAULT		1000
#define	OUTPUTNAMEDEFAULT	"x"

char	inbuf[16*1024];
char	outbuf[31*1024];
char	linebuff[LINE_SIZE];
char	outname[LPNMAX];
char	soutname[LPNMAX] = OUTPUTNAMEDEFAULT;

main(argc, argv)
int	argc;
char	**argv;
{
	FILE	*fi;
	int	linecount = NUMLINESDEFAULT;

	/** "Usage: split [-n] [file [name]]\n" **/
	SHIFT;
	if (argc > 0 && **argv == '-' && strlen(*argv) > 1) {
		linecount = atoi(&(*argv)[1]);
		SHIFT;
	}
	if (linecount <= 0)
		usage();
	fi = stdin;
	if (argc > 0) {
		if (0 == strcmp(*argv, "-")) {
			fi = stdin;
		} else {
			if ((FILE *)NULL == (fi = fopen(*argv, "r"))) {
				printf("%s: cannot open\n", *argv);
				exit(2);
			}
		}
		SHIFT;
	}
	if (argc > 0) {
		strcpy(soutname, *argv);
		SHIFT;
	}
	if (argc > 0)
		usage();

	if (0 != setvbuf(fi, inbuf, _IOFBF, sizeof(inbuf))) {
		printf("cannot set input buffer\n");
		exit(4);
	}
	split(fi, linecount);
	fclose(fi);
	exit(0);
}

split(fi, linecount)
FILE	*fi;
int	linecount;
{
	int	fcount = 0;	/* file count	*/
	int	lcount;		/* line count	*/
	FILE	*fo;
	int	done = 0;

	while (!done) {
		makeoutname(fcount);
		if ((FILE *)NULL == (fo = fopen(outname, "w"))) {
			printf("%s: cannot open\n", outname);
			exit(2);
		}
		if (0 != setvbuf(fo, outbuf, _IOFBF, sizeof(outbuf))) {
			printf("cannot set output buffer\n");
			exit(4);
		}
		lcount = 0;
		while (lcount++ < linecount) {
			if ((char *)NULL == fgets(linebuff, LINE_SIZE, fi)) {
				done++;
				break;
			}
			if (EOF == fputs(linebuff, fo)) {
				printf("%s: write error\n", outname);
				exit(3);
			}
		}
		fclose(fo);
		fcount++;
	}
}

/*
 *  makeoutname  --  create output file name in "outname" from name
 *			base in "soutname" and count in the 
 *			fashion aa, ab, ac, ..., az, ba, bb, ..., zz.
 */
makeoutname(count)
int	count;
{
	char	*s;

	strcpy(outname, soutname);
	outname[BASE_NAME_LEN - 2 - 1] = '\0';
	s = &outname[strlen(outname)];
	*s++ = 'a' + (count / 26);
	*s++ = 'a' + (count % 26);
	*s   = '\0';
}

usage()
{
	printf("Usage: split [-n] [input_file [output_name]]\n");
	printf("   Splits input into a series of smaller files, each a\n");
	printf("   specified number of lines long.\n");
	printf("   Output filenames are generated with a base filename\n");
	printf("   followed by a two letter suffix, in the order:\n");
	printf("      aa, ab, ac, ..., az, ba, bb, bc, ...\n");
	printf("  -n:          input is split in n line parts, default = %d\n",
		NUMLINESDEFAULT);
	printf("  input_file:  input file or \"-\" for standard input\n");
	printf("  output_name: character string with which to start\n");
	printf("               output file names, default = \"%s\"\n",
		OUTPUTNAMEDEFAULT);
	exit(1);
}
